@@ -1,4 +1,4 @@
-# Copyright 2018 Jeffrey Kegler
+# Copyright 2022 Jeffrey Kegler
 # This file is part of Marpa::R2.  Marpa::R2 is free software: you can
 # redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation,
@@ -20,7 +20,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '8.000000';
+$VERSION        = '9.001_000';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -1922,6 +1922,29 @@ sub Marpa::R2::Scanless::R::earley_set_size {
         ->earley_set_size($set_id);
 }
 
+sub Marpa::R2::Scanless::R::verbose_or_nodes {
+    my ( $slr ) = @_;
+    my $thick_g1_recce =
+        $slr->[Marpa::R2::Internal::Scanless::R::THICK_G1_RECCE];
+    return $thick_g1_recce->verbose_or_nodes();
+}
+
+sub Marpa::R2::Scanless::R::show_and_nodes {
+    my ( $slr, $verbose ) = @_;
+    $verbose //= 0;
+    my $thick_g1_recce =
+        $slr->[Marpa::R2::Internal::Scanless::R::THICK_G1_RECCE];
+    return $thick_g1_recce->show_and_nodes($verbose);
+}
+
+sub Marpa::R2::Scanless::R::show_bocage {
+    my ( $slr, $verbose ) = @_;
+    $verbose //= 0;
+    my $thick_g1_recce =
+        $slr->[Marpa::R2::Internal::Scanless::R::THICK_G1_RECCE];
+    return $thick_g1_recce->show_bocage($verbose);
+}
+
 1;
 
 # vim: expandtab shiftwidth=4:
