@@ -1,4 +1,4 @@
-# Copyright 2013 Jeffrey Kegler
+# Copyright 2022 Jeffrey Kegler
 # This file is part of Marpa::R2.  Marpa::R2 is free software: you can
 # redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation,
@@ -15,7 +15,7 @@
 
 package Marpa::R2::Perl;
 
-use 5.010;
+use 5.010001;
 use strict;
 use warnings;
 
@@ -978,7 +978,7 @@ sub Marpa::R2::Perl::new {
 } ## end sub Marpa::R2::Perl::new
 
 my @RECCE_NAMED_ARGUMENTS =
-    qw(trace_tasks trace_terminals trace_values trace_actions);
+    qw(trace_terminals trace_values trace_actions);
 
 sub token_not_accepted {
     my ( $ppi_token, $token_name, $token_value, $length ) = @_;
@@ -1056,13 +1056,14 @@ sub Marpa::R2::Perl::read_tokens {
     $hash_arg //= {};
 
     my @recce_args = ();
-    HASH_ARG: while ( my ( $arg, $value ) = each %{$hash_arg} ) {
+    HASH_ARG: for my $arg ( keys %{$hash_arg} ) {
+        my $value = $hash_arg->{$arg};
         if ( grep { $_ eq $arg } @RECCE_NAMED_ARGUMENTS ) {
             push @recce_args, $arg, $value;
             next HASH_ARG;
         }
         Carp::croak("Unknown hash arg: $arg");
-    } ## end HASH_ARG: while ( my ( $arg, $value ) = each %{$hash_arg} )
+    } ## end HASH_ARG: for my $arg ( keys %{$hash_arg} )
 
     my $grammar = $parser->{grammar};
 
@@ -1226,11 +1227,11 @@ sub read_PPI_token {
     if ( $PPI_type eq 'PPI::Token::Operator' ) {
         my $content = $token->{content};
         $perl_type = $perl_type_by_op{$content};
-	if (not defined $perl_type) {
-	  unknown_ppi_token($token);
-	  $parser->earleme_complete();
-	  goto SUCCESS;
-	}
+        if (not defined $perl_type) {
+          unknown_ppi_token($token);
+          $parser->earleme_complete();
+          goto SUCCESS;
+        }
         if ( $perl_type eq 'PLUS' ) {
 
             # Apply the "ruby slippers"
@@ -1278,11 +1279,11 @@ sub read_PPI_token {
         my $content = $token->{content};
         $perl_type = $perl_type_by_structure{$content};
         my $expected_tokens = $parser->{terminals_expected};
-	if ( not defined $perl_type ) {
-	    unknown_ppi_token($token);
-	    $parser->earleme_complete();
-	    goto SUCCESS;
-	}
+        if ( not defined $perl_type ) {
+            unknown_ppi_token($token);
+            $parser->earleme_complete();
+            goto SUCCESS;
+        }
         if ( $perl_type eq 'RCURLY' ) {
             if ((   not defined $Marpa::R2::Perl::LAST_PERL_TYPE
                     or $Marpa::R2::Perl::LAST_PERL_TYPE ne 'SEMI'
@@ -1413,10 +1414,10 @@ sub Marpa::R2::Perl::find_perl {
         )
     {
         if ($recce->exhausted())
-	{
-	  die 'Exhausted but no program found\?' if not defined $last_end_marker_ix;
-	  last TOKEN;
-	}
+        {
+          die 'Exhausted but no program found\?' if not defined $last_end_marker_ix;
+          last TOKEN;
+        }
         my $current_earleme = $recce->current_earleme();
         $earleme_to_PPI_token->[$current_earleme] //= $PPI_token_ix;
         $PPI_token_to_earleme[$PPI_token_ix] = $current_earleme;
@@ -1451,10 +1452,10 @@ sub Marpa::R2::Perl::find_perl {
     my $start;
     ITEM: for my $item (@{$report}) {
         my ($rule_id, $dot_position, $origin) = @{$item};
-	next ITEM if $dot_position >= 0;
-	next ITEM if ($grammar->rule($rule_id))[0] ne 'prog';
-	$start //= $origin;
-	$start = $origin if $start > $origin;
+        next ITEM if $dot_position >= 0;
+        next ITEM if ($grammar->rule($rule_id))[0] ne 'prog';
+        $start //= $origin;
+        $start = $origin if $start > $origin;
     }
     die 'End marker, but no Perl prog?' if not defined $start;
     my $start_PPI_ix = $earleme_to_PPI_token->[$start];
@@ -1496,3 +1497,5 @@ sub Marpa::R2::Perl::default_show_location {
 } ## end sub Marpa::R2::Perl::default_show_location
 
 1;
+
+# vim: set expandtab shiftwidth=4:
